@@ -54,6 +54,7 @@ EOT
 $config .= join("\n", @config) if @config;
 $config .= join("", map { "use App::Phoebe::$_;\n" } @use) if @use;
 if ($example) {
+  my $found;
   for my $file ("blib/lib/App/Phoebe.pm", map { "blib/lib/App/Phoebe/$_" } grep /\.pm$/, read_dir("blib/lib/App/Phoebe")) {
     my $source = read_text($file);
     if ($source =~ /^(    # tested by $0\n(?:    .*\n|\t.*\n|\n)+)/m) {
@@ -61,9 +62,11 @@ if ($example) {
       $example =~ s/\t/        /g;
       $example =~ s/^    //gm;
       $config .= $example;
+      $found = 1;
       last;
     }
   }
+  ok($found, "$0 found");
   # test $example at the end
 }
 write_text("$dir/config", $config . "\n1;\n") if $config;
@@ -103,16 +106,17 @@ sub query_gemini {
   my $query = shift;
   my $text = shift;
   my $cert = shift // 1; # suppress use of client certificate in the test
+  my $cert_file = [undef, "t/cert.pem", "t/cert2.pem"]->[$cert];
+  my $key_file = [undef, "t/key.pem", "t/key2.pem"]->[$cert];
   my ($header, $mimetype, $encoding, $buffer);
-
   # create client
   Mojo::IOLoop->client(
     {
       address => "localhost",
       port => $port,
       tls => 1,
-      tls_cert => ($cert ? "t/cert.pem" : undef),
-      tls_key => ($cert ? "t/key.pem" : undef),
+      tls_cert => $cert_file,
+      tls_key => $key_file,
       tls_options => { SSL_verify_mode => 0x00 },
     } => sub {
       my ($loop, $err, $stream) = @_;
@@ -161,7 +165,16 @@ sub query_web {
   my $query = shift;
   my $cert = shift // 1; # suppress use of client certificate in the test
   $query .= "\r\n" unless $query =~ /^POST/; # add empty line for GET requests
-  return query_gemini($query, undef, $cert);
+  my $response = query_gemini($query, undef, $cert);
+  # fixup encoding for two trivial cases of encoding html
+  my $header_end = index($response, "\r\n\r\n");
+  if (substr($response, 0, $header_end + 2) =~ /content-type: text\/[a-z]+; charset=(\S+)/i
+      or substr($response, $header_end + 4) =~ /<meta charset=\"(\S+)\">/i) {
+    my $encoding = $1;
+    $response = substr($response, 0, $header_end + 4)
+	. decode($encoding, substr($response, $header_end + 4));
+  }
+  return $response;
 }
 
 my $total = 0;
@@ -170,7 +183,9 @@ my $ok = 0;
 # What I'm seeing is that $@ is the empty string and $! is "Connection refused"
 # even though I thought $@ would be set. Oh well.
 say "This is the Phoebe client waiting for the server to start on port $port...";
-for (qw(1 1 1 1 1)) {
+# In order to avoid "skipped: Giving up after 5s" by CPAN tester gregor herrmann,
+# make sure to wait more than that!
+for (qw(1 1 1 1 1 2 3 5)) {
   if (not $total or $!) {
     $total += $_;
     sleep $_;
